@@ -7,9 +7,12 @@ from time import gmtime, strftime
 import glob
 
 import argparse
+import astropy.coordinates as coord
 
 import logging
 from subprocess import Popen, PIPE, STDOUT
+from astroquery.simbad import Simbad
+import yaml
 
 # file_handler = logging.FileHandler(filename='nustar_process_%s.log' % (strftime("%Y-%m-%dT%H:%M:%S", gmtime())))
 # stdout_handler = logging.StreamHandler(sys.stdout)
@@ -109,6 +112,12 @@ def write_region(fname, ra, dec, src_flag, ra_back=-1, dec_back=-1, radius=120):
 
 
 def get_src_from_obsid(obsid, repository_path=os.environ['HOME'] + '/NUSTAR/Repository'):
+	'''
+	Gets the source and the coordinates from the data repository
+	:param obsid:
+	:param repository_path:
+	:return:
+	'''
 	from astropy.io import fits as pf
 	search_str = repository_path + '/' + obsid + '/event_cl/nu*A*_cl.evt.gz'
 	logger.debug(search_str)
@@ -118,8 +127,48 @@ def get_src_from_obsid(obsid, repository_path=os.environ['HOME'] + '/NUSTAR/Repo
 		return None
 	with pf.open(eventsA[0]) as ff:
 		src = ff[1].header['OBJECT']
+		ra = ff[1].header['RA_OBJ']
+		dec = ff[1].header['DEC_OBJ']
 	logger.debug('Found source ' + src)
-	return src
+	return src, ra, dec
+
+def prepare_folder_from_obsid(oo, output_base_folder='.'):
+	'''
+	Prepare a folder to process data
+	:param oo: the obsid
+	:return: None
+	'''
+
+	src, ra, dec = get_src_from_obsid(oo)
+	result_table = Simbad.query_region(coord.SkyCoord(ra,dec, unit='deg', frame='icrs'), radius='1m')
+	all_names = Simbad.query_objectids(result_table['MAIN_ID'][0])
+
+	try:
+		os.mkdir(output_base_folder+'/'+src)
+	except OSError as error:
+		print(error)
+
+	workdir = output_base_folder+'/'+src+'/'+oo
+
+	try:
+		os.mkdir(workdir)
+	except OSError as error:
+		print(error)
+
+	src_dict = {
+		'OBJECT': src,
+		'RA': ra,
+		'DEC': dec,
+		'NAMES': [str(ss) for ss in all_names['ID']],
+		'OBSID': oo
+	}
+
+	with open(workdir + '/observation.yml', 'w') as outfile:
+		yaml.dump(src_dict, outfile, default_flow_style=False)
+
+	print("Processed " + src + ' with obsid ' + oo)
+
+
 
 def wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag=False, region_flag=False,
 				 spec_flag=False, lc_flag=False,
