@@ -1,9 +1,8 @@
-import sys, os
-from time import gmtime, strftime
-import numpy as np
 import logging
 from numpy import random
 import copy
+import numpy as np
+import matplotlib.pyplot as plt
 
 # file_handler = logging.FileHandler(filename='nustar_utils_%s.log' % (strftime("%Y-%m-%dT%H:%M:%S", gmtime())))
 # stdout_handler = logging.StreamHandler(sys.stdout)
@@ -598,7 +597,7 @@ def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=Fa
                 dnew_pulses.append(dp1/float(i2-i1))
                 new_e_mins.append(e_min[i1])
                 new_e_maxs.append(e_max[i2 - 1])
-                logger.debug("Boom %f %d %d " % (s_n, i1, i2) )
+                logger.debug("Boom %f %d %d " % (s_n, i1, i2))
                 # print("Boom", s_n, i1, i2)
                 # print(np.mean(p1)/float(i2-i1), np.mean(dp1)/float(i2-i1))
                 i1 = i2
@@ -678,6 +677,134 @@ def pulse_fraction_from_data_rms(counts, counts_err, n_harm=3):
     return (PF_rms)
 
 
+#this function will fit an input energy range of the pulse profile.
+#the fitting function will be a simple polynomial + gaussian
+#the aim is to retrieve basic gaussian parameters to be compared
+#to those obtained in spectral analysis around Ecycl.
+# output files: 1. the fit result
+#               2. the fit results to be plotted in a file
+#               3. Figure in pdf
+
+
+
+def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center=[],
+                    sigma=[], amplitude=[],
+                    plot_final=True, print_results=True):
+    '''
+    this function will fit an input energy range of the pulse profile.
+    the fitting function will be a simple polynomial + gaussian
+    the aim is to retrieve basic gaussian parameters to be compared
+    to those obtained in spectral analysis around Ecycl.
+     output files: 1. the fit result
+                   2. the fit results to be plotted in a file
+                   3. Figure in pdf
+    :param en: input energy
+    :param den: input energy uncertainty
+    :param pf: pulsed fraction
+    :param dpf: pulsed fractio uncertainty
+    :param stem: output prefix
+    :param degree_pol: degree of the polynomial to fit
+    :param n_gauss: number of gaussian lines
+    :param center: array of centers of gaussians [[initial_value, min, max]]
+    :param sigma: array of sigmas of gaussians [[initial_value, min, max]]
+    :param amplitude: array of amplitude of gaussians [[initial_value, min, max]],
+                        use negative values for absorption-like
+    :param plot_final: if make a final plot
+    :param print_results: if results should be printed out in file
+    :return:
+    '''
+    from matplotlib.pyplot import cm
+    from lmfit.models import PolynomialModel, GaussianModel
+
+    if len(center) != n_gauss or len(sigma) != n_gauss or len (amplitude) != n_gauss:
+        logger.error("You should provide centers, sigmas, and amplitudes for %d gaussians" % n_gauss)
+        return
+
+    col = cm.viridis(np.linspace(0, 1, 6))
+
+    if stem is not None:
+        outputfile = open(stem + '_fit_pf.out', 'w')
+    else:
+        stem = ''
+        outputfile = open('fit_pf.out', 'w')
+
+    poly_mod = PolynomialModel(prefix='poly_', degree=degree_pol)
+    pars = poly_mod.guess(pf, x=en, degree=degree_pol)
+    mod = poly_mod
+
+    for N in range(1, n_gauss+1):
+        logger.debug("%d" % N)
+        logger.debug("%g" % center[N - 1][0])
+        gauss = GaussianModel(prefix='g' + str(N) + '_')
+        pars.update(gauss.make_params())
+        pars['g' + str(N) + '_center'].set(value=center[N - 1][0], min=center[N - 1][1], max=center[N - 1][2])
+        pars['g' + str(N) + '_sigma'].set(sigma[N - 1][0], min=sigma[N - 1][1], max=sigma[N - 1][2])
+        pars['g' + str(N) + '_amplitude'].set(amplitude[N - 1][0], min=amplitude[N - 1][1],
+                                               max=amplitude[N - 1][2])
+        mod = mod + gauss
+
+    #initialfit = mod.eval(pars, x=en)
+
+    out = mod.fit(pf, pars, x=en)
+
+    bb = (pf - out.best_fit) / dpf
+
+    comps = out.eval_components(x=en)
+    logger.info(comps)
+
+    if not plot_final:
+        fig, axes = plt.subplots(1, 2, figsize=(8.8, 4.8))
+        axes[0].errorbar(en, pf, xerr=den, yerr=dpf, fmt='.', color=col[0])
+        axes[0].plot(en, out.best_fit, '-', label='best fit', color=col[2])
+        axes[0].legend(loc='upper left')
+        axes[0].set_xlabel('E [keV]')
+        axes[0].set_ylabel('PF')
+
+        axes[1].errorbar(en, pf, xerr=den, yerr=dpf, fmt='.', color=col[0])
+        for N in range(1, n_gauss+1):
+            axes[1].plot(en, comps['g%d_' % N], '--', label='Gaussian %d' % N, color=col[3])
+        axes[1].plot(en, comps['poly_'], '--', label='Polynomial component', color=col[4])
+        axes[1].legend(loc='upper left')
+        axes[1].set_xlabel('E [keV]')
+        axes[1].set_ylabel('PF')
+        plt.savefig(stem + 'fit_results.pdf')
+
+    if print_results:
+        datafile = open(stem + 'model_components_fit.dat', 'w')
+        datafile.write('# fit results \n')
+        datafile.write('# E[0]       dE[1]       pf_bestfit[2]          poly[3]            gauss1...N [4]...[N+4] \n')
+        for j in range(len(pf)):
+            datafile.write(str(round(en[j], 5)).ljust(10) + str(round(den[j], 2)).ljust(10) +
+                           str(round(out.best_fit[j], 5)).ljust(15) + str(round(comps['poly_'][j], 5)).ljust(15) + '\t')
+            for S in range(1, n_gauss+1):
+                datafile.write(str(round(comps['g' + str(S) + '_'][j], 8)).ljust(25) + '\t')
+            datafile.write('\n')
+
+        datafile.close()
+
+    if plot_final:
+        fig, (ax1, ax2) = plt.subplots(2, 1, figsize=(6.2, 6.0), sharex=True, gridspec_kw={'height_ratios': [3, 1],
+                                                                              'hspace': 0.0}
+                                       )
+        ax1.errorbar(en, pf, xerr=den, yerr=dpf, fmt='.', color=col[4], label='data')
+        ax1.plot(en, out.best_fit, '-', label='best fit', color=col[0])
+        ax1.plot(en, comps['poly_'], '--', label='Polynomial', color=col[1])
+        ax1.legend(loc='upper left')
+        ax2.set_xlabel('E (keV)')
+        ax1.set_ylabel('PF')
+        ax2.set_ylabel('Residuals')
+        ax1.set_ylim(-0.1, 1)
+        ax2.errorbar(en, bb, xerr=den, yerr=1., fmt='.', color=col[2])
+        ax2.axhline(y=0, color='k', linestyle='--')
+        plt.savefig(stem + 'pulsed_fitted.pdf')
+
+    logger.info(str(out.fit_report()))
+    outputfile.write(out.fit_report())
+    outputfile.close()
+
+    return out
+
+
 def get_error_from_simul_rms(counts, counts_err, n_simul=100, n_harm=3):
 
     return get_error_from_simul(counts, counts_err, pulse_fraction_from_data_rms,
